from waveguide import Waveguide
import sbend
import fdtd

# import os


class DirectionalCoupler(Waveguide):
    """方向耦合器，返回两个s波导的介电常数矩阵
    xlength: 波导区域x方向宽度
    ylength: 波导区域y方向宽度
    zlength: 波导区域z方向宽度，通常也是波导高度
    x,y,z: 波导位置坐标（通常是矩形区域最靠近原点的点）
    flag：参数
    width：波导宽度
    refractive_index:折射率
    gap:直波导间距
    xlength_rectangle：直波导长度(耦合长度)"""

    # TODO 只需要输出两个s波导的矩阵，另外两个S波导的矩阵与他们一样，关键在于确定位置（未完成）
    def __init__(
        self,
        xlength=60,
        ylength=10,
        zlength=10,
        x=50,
        y=50,
        z=50,
        flag=1,
        width=10,
        name="waveguide",
        refractive_index=1.7,
        xlength_rectangle=30,
        gap=5,
    ):
        Waveguide.__init__(self, xlength, ylength, zlength, x, y, z, flag, width, name, refractive_index)
        self.xlength_rectangle = xlength_rectangle
        self.ylength_sbend = int((ylength - gap) / 2 + 0.5)
        self.xlength_sbend = int((xlength - xlength_rectangle) / 2 + 0.5)
        self.gap = gap

    def set_grid(self):
        # permittivity_rectangle = 应该不用写矩形波导的介电常数？
        # 左上波导sbend1
        sbend1 = sbend.Sbend(
            xlength=self.xlength_sbend,
            ylength=self.ylength_sbend,
            zlength=self.zlength,
            x=self.x,
            y=self.y,
            z=self.z,
            flag=1,
            width=self.width,
            refractive_index=self.refractive_index,
            name="DC_sbend1",
        )
        # 左下波导sbend2
        sbend2 = sbend.Sbend(
            xlength=self.xlength_sbend,
            ylength=self.ylength_sbend,
            zlength=self.zlength,
            x=self.x + self.xlength_sbend + self.xlength_rectangle,
            y=self.y,
            z=self.z,
            flag=-1,
            width=self.width,
            refractive_index=self.refractive_index,
            name="DC_sbend2",
        )

        # 右上波导sbend3
        sbend3 = sbend.Sbend(
            xlength=self.xlength_sbend,
            ylength=self.ylength_sbend,
            zlength=self.zlength,
            x=self.x,
            y=self.y + self.ylength_sbend + self.gap,
            z=self.z,
            flag=-1,
            width=self.width,
            refractive_index=self.refractive_index,
            name="DC_sbend3",
        )

        # 右下波导sbend4
        sbend4 = sbend.Sbend(
            xlength=self.xlength_sbend,
            ylength=self.ylength_sbend,
            zlength=self.zlength,
            x=self.x + self.xlength_sbend + self.xlength_rectangle,
            y=self.y + self.ylength_sbend + self.gap,
            z=self.z,
            flag=1,
            width=self.width,
            refractive_index=self.refractive_index,
            name="DC_sbend4",
        )

        wg1 = Waveguide(
            xlength=self.xlength_rectangle,
            ylength=self.width,
            zlength=self.zlength,
            x=self.x + self.xlength_sbend,
            y=self.y + self.ylength_sbend - self.width,
            z=self.z,
            flag=-1,
            width=self.width,
            refractive_index=self.refractive_index,
            name="DC_wg1",
        )

        wg2 = Waveguide(
            xlength=self.xlength_rectangle,
            ylength=self.width,
            zlength=self.zlength,
            x=self.x + self.xlength_sbend,
            y=self.y + self.ylength_sbend + self.gap,
            z=self.z,
            flag=-1,
            width=self.width,
            refractive_index=self.refractive_index,
            name="DC_wg2",
        )

        # result_sbend1 = sbend1.set_grid()
        # result_sbend2 = sbend2.set_grid()
        # result_sbend3 = sbend3.set_grid()
        # result_sbend4 = sbend4.set_grid()
        # result_wg1 = wg1.set_grid()
        # result_wg2 = wg2.set_grid()

        result = (
            sbend1.set_grid(),
            sbend2.set_grid(),
            sbend3.set_grid(),
            sbend4.set_grid(),
            wg1.set_grid(),
            wg2.set_grid(),
        )

        return result


if __name__ == "__main__":
    dc = DirectionalCoupler(
        xlength=150,
        ylength=65,
        zlength=1,
        x=20,
        y=20,
        z=20,
        flag=1,
        width=4,
        name="dc",
        refractive_index=3.47,
        xlength_rectangle=60,
        gap=1,
    )
    result = dc.set_grid()
    # print(p1, p2)

    grid = fdtd.Grid(shape=(175, 120, 1), grid_spacing=155e-9, permittivity=1**2)

    for i in range(6):
        grid[
            result[i]["position"][0] : result[i]["position"][0] + result[i]["size"][0],
            result[i]["position"][1] : result[i]["position"][1] + result[i]["size"][1],
        ] = fdtd.Object(permittivity=result[i]["permittivity"], name=result[i]["name"])

    PML_width = 5

    grid[0:PML_width, :, :] = fdtd.PML(name="pml_xlow")
    grid[-PML_width:, :, :] = fdtd.PML(name="pml_xhigh")
    grid[:, 0:PML_width, :] = fdtd.PML(name="pml_ylow")
    grid[:, -PML_width:, :] = fdtd.PML(name="pml_yhigh")

    grid[10:10, result[0]["position"][1] : result[0]["position"][1] + dc.width] = fdtd.LineSource(
        period=1550e-9 / 299792458, name="source", pulse=True
    )
    # grid[10:10, 10:110] = fdtd.LineSource(
    #     period=1550e-9 / 299792458, name="source")

    # grid[55:55, 28:28] = fdtd.PointSource(
    # period=5.16e-15, name="sourced")

    # grid[10:110, 10:110, 0] = fdtd.BlockDetector(name="detector")

    # simfolder = grid.save_simulation("dc")  # initializing environment to save simulation data

    # with open(os.path.join(simfolder, "grid.txt"), "w") as f:
    #     f.write(str(grid))
    #     wavelength = 1550e-9
    #     wavelengthUnits = wavelength/grid.grid_spacing
    #     GD = np.array([grid.x, grid.y, grid.z])
    #     gridRange = [np.arange(x/grid.grid_spacing) for x in GD]
    #     objectRange = np.array([[gridRange[0][x.x], gridRange[1][x.y], gridRange[2][x.z]] for x in grid.objects], dtype=object).T
    #     f.write("\n\nGrid details (in wavelength scale):")
    #     f.write("\n\tGrid dimensions: ")
    #     f.write(str(GD/wavelength))
    #     # f.write("\n\tSource dimensions: ")
    #     # f.write(str(np.array([grid.source.x[-1] - grid.source.x[0] + 1, grid.source.y[-1] - grid.source.y[0] + 1, grid.source.z[-1] - grid.source.z[0] + 1])/wavelengthUnits))
    #     f.write("\n\tObject dimensions: ")
    #     f.write(str([(max(map(max, x)) - min(map(min, x)) + 1)/wavelengthUnits for x in objectRange]))

    grid.run(total_time=1000)
    # grid.save_data()

    # df = np.load(os.path.join(simfolder, "detector_readings.npz"))
    # fdtd.dB_map_2D(df["detector (E)"])
    grid.visualize(z=0, show=True)



from mmi import Mmi
import fdtd

# import os


if __name__ == "__main__":
    n = 1  # 输入端口数
    m = 2  # 输出端口数
    time = 1500
    PML_width = 5
    lambda0 = 1550e-9  # 输入光波长（单位m）
    grid_spacing = 110e-9  # 空间步长

    mmi = Mmi(
        xlength=118,
        ylength=36,
        zlength=1,
        We=37,
        x=50,
        y=10,
        z=1,
        flag=1,
        name="mmi",
        refractive_index=3.47,
        n=n,
        m=m,
        width_port=8,
        width_wg=2,
        l_port=10,
        ln=40,
        lm=5,
    )

    result0 = mmi.set_box()
    result_in, result_out, result_in_wg, result_out_wg = mmi.set_ports()
    grid = fdtd.Grid(
        shape=(mmi.xlength + mmi.ln + mmi.lm + mmi.l_port * 2, mmi.ylength + 2 * PML_width + 10, 1),
        grid_spacing=grid_spacing,
        permittivity=1,
    )

    for i in range(n):
        grid[
            result_in[i]["position"][0] : result_in[i]["position"][0] + result_in[i]["size"][0],
            result_in[i]["position"][1] : result_in[i]["position"][1] + result_in[i]["size"][1],
            :,
        ] = fdtd.Object(permittivity=result_in[i]["permittivity"], name=result_in[i]["name"])

        grid[
            result_in_wg[i]["position"][0] : result_in_wg[i]["position"][0] + result_in_wg[i]["size"][0],
            result_in_wg[i]["position"][1] : result_in_wg[i]["position"][1] + result_in_wg[i]["size"][1],
            :,
        ] = fdtd.Object(permittivity=result_in_wg[i]["permittivity"], name=result_in_wg[i]["name"])

    for i in range(m):
        grid[
            result_out[i]["position"][0] : result_out[i]["position"][0] + result_out[i]["size"][0],
            result_out[i]["position"][1] : result_out[i]["position"][1] + result_out[i]["size"][1],
            :,
        ] = fdtd.Object(permittivity=result_out[i]["permittivity"], name=result_out[i]["name"])

        grid[
            result_out_wg[i]["position"][0] : result_out_wg[i]["position"][0] + result_out_wg[i]["size"][0],
            result_out_wg[i]["position"][1] : result_out_wg[i]["position"][1] + result_out_wg[i]["size"][1],
            :,
        ] = fdtd.Object(permittivity=result_out_wg[i]["permittivity"], name=result_out_wg[i]["name"])

    grid[
        result0["position"][0] : result0["position"][0] + result0["size"][0],
        result0["position"][1] : result0["position"][1] + result0["size"][1],
        :,
    ] = fdtd.Object(permittivity=result0["permittivity"], name=result0["name"])

    grid[0:PML_width, :, :] = fdtd.PML(name="pml_xlow")
    grid[-PML_width:, :, :] = fdtd.PML(name="pml_xhigh")
    grid[:, 0:PML_width, :] = fdtd.PML(name="pml_ylow")
    grid[:, -PML_width:, :] = fdtd.PML(name="pml_yhigh")

    for i in range(n):
        grid[
            9:9, result_in[i]["position"][1] : result_in[i]["position"][1] + result_in[i]["size"][1], :
        ] = fdtd.LineSource(period=1550e-9 / 299792458, name="source%d" % i)

    # grid[result0['position'][0]:result0['position'][0]+result0['size'][0],
    # result0['position'][1]:result0['position'][1]+result0['size'][1], 0] = fdtd.BlockDetector(name="detector")
    #
    # simfolder = grid.save_simulation("mmi")  # initializing environment to save simulation data
    #
    # with open(os.path.join(simfolder, "grid.txt"), "w") as f:
    #     f.write(str(grid))
    #     wavelength = 1550e-9
    #     wavelengthUnits = wavelength/grid.grid_spacing
    #     GD = np.array([grid.x, grid.y, grid.z])
    #     gridRange = [np.arange(x/grid.grid_spacing) for x in GD]
    #     objectRange = np.array([[gridRange[0][x.x], gridRange[1][x.y], gridRange[2][x.z]] for x in grid.objects], dtype=object).T
    #     f.write("\n\nGrid details (in wavelength scale):")
    #     f.write("\n\tGrid dimensions: ")
    #     f.write(str(GD/wavelength))
    #     # f.write("\n\tSource dimensions: ")
    #     # f.write(str(np.array([grid.source.x[-1] - grid.source.x[0] + 1, grid.source.y[-1] - grid.source.y[0] + 1, grid.source.z[-1] - grid.source.z[0] + 1])/wavelengthUnits))
    #     f.write("\n\tObject dimensions: ")
    #     f.write(str([(max(map(max, x)) - min(map(min, x)) + 1)/wavelengthUnits for x in objectRange]))

    grid.run(total_time=time)
    # grid.save_data()

    # df = np.load(os.path.join(simfolder, "detector_readings.npz"))
    # fdtd.dB_map_2D(df["detector (E)"])

    grid.visualize(z=0, show=True)



from .waveguide import Waveguide
from . import sbend
import numpy as np
import fdtd


class Trapezoid(Waveguide):
    """梯形（锥形）耦合部分代码，继承自waveguide
    xlength: 多模波导区域水平方向宽度 (在python fdtd包中, x方向为竖直方向，y为水平方向)
    ylength: 多模波导区域竖直方向宽度
    zlength: 多模波导区域z方向宽度，通常也是波导高度
    x,y,z: 多模波导位置坐标（通常是矩形区域最靠近原点的点）
    direction: 表示方向, direction = 1表示窄口在上（x轴负方向），direction = -1表示窄口在下（x轴正方向）
    width：窄处宽度
    refractive_index:折射率"""

    def __init__(
        self,
        xlength=60,
        ylength=10,
        zlength=10,
        x=50,
        y=50,
        z=50,
        direction=1,
        width=10,
        name="waveguide",
        refractive_index=1.7,
    ):

        self.direction = direction
        super().__init__(xlength, ylength, zlength, x, y, z, width, name, refractive_index)

    def _compute_permittivity(self):
        """输入波导规格，返回介电常数矩阵
        width:波导窄处宽度
        ylength:波导宽处宽度"""
        x = np.linspace(0, self.xlength, self.xlength)
        y = np.linspace(0, self.ylength, self.ylength)
        X, Y = np.meshgrid(x, y, indexing="ij")

        m = np.zeros((self.xlength, self.ylength, 1))

        if self.direction == 1:
            # 开口向下（x正方向）
            for i in range(self.xlength):
                for j in range(self.ylength):
                    if (
                        X[i, j] * (self.width / 2 - self.ylength / 2) / self.xlength
                        + self.ylength / 2
                        - self.width / 2
                        <= Y[i, j]
                        <= X[i, j] * (-self.width / 2 + self.ylength / 2) / self.xlength
                        + self.ylength / 2
                        + self.width / 2
                    ):
                        m[i, j] = True

        if self.direction == -1:
            # 开口向上（x负方向）
            for i in range(self.xlength):
                for j in range(self.ylength):
                    if (
                        X[self.xlength - i - 1, j] * (self.width / 2 - self.ylength / 2) / self.xlength
                        + self.ylength / 2
                        - self.width / 2
                        <= Y[i, j]
                        <= X[self.xlength - i - 1, j] * (-self.width / 2 + self.ylength / 2) / self.xlength
                        + self.ylength / 2
                        + self.width / 2
                    ):
                        m[i, j] = True

        permittivity = np.ones((self.xlength, self.ylength, self.zlength))
        permittivity += m[:, :] * (self.refractive_index**2 - 1)

        self.permittivity = permittivity


class Ysplitter(Waveguide):

    """Y分支波导，由一段直波导，一个梯形，两个S波导组成"""

    def __init__(
        self,
        xlength=60,
        ylength=10,
        zlength=10,
        x=50,
        y=50,
        z=50,
        direction=-1,
        width=10,
        name="ysplitter",
        refractive_index=1.7,
        xlength_rectangle=30,
        xlength_trapezoid=10,
        ylength_trapezoid=30,
    ):
        super().__init__(xlength, ylength, zlength, x, y, z, width, name, refractive_index)
        self.direction = direction
        self.xlength_rectangle = xlength_rectangle
        self.xlength_trapezoid = xlength_trapezoid
        self.ylength_trapezoid = ylength_trapezoid
        self.xlength_sbend = xlength - xlength_rectangle - xlength_trapezoid
        self.ylength_sbend = int(ylength / 2 - ylength_trapezoid / 2 + width + 0.5)

    def set_grid(
        self, grid_ylength=80, grid_xlength=80, grid_zlength=1, grid_spacing=155e-9, total_time=200, pml_width=10
    ):

        """返回四个部分的名称、介电常数矩阵、规格、位置，分别为直波导、梯形、s波导1、s波导2"""
        if self.direction == 1:
            waveguide = Waveguide(
                xlength=self.xlength_rectangle,
                ylength=self.width,
                zlength=self.zlength,
                x=self.x,
                y=self.y + int(self.ylength / 2 - self.width / 2 + 0.5),
                z=self.z,
                width=self.width,
                name="%s_waveguide" % self.name,
            )
            # waveguide.set_grid()
            trapezoid = Trapezoid(
                xlength=self.xlength_trapezoid,
                ylength=self.ylength_trapezoid,
                zlength=self.zlength,
                x=self.x + self.xlength_rectangle,
                y=self.y + int(self.ylength / 2 - self.ylength_trapezoid / 2 + 0.5),
                z=self.z,
                direction=self.direction,
                width=self.width,
                name="%s_trapezoid" % self.name,
            )
            # trapezoid.set_grid()
            sbend1 = sbend.Sbend(
                xlength=self.xlength_sbend,
                ylength=self.ylength_sbend,
                zlength=self.zlength,
                x=self.x + self.xlength_rectangle + self.xlength_trapezoid,
                y=self.y,
                z=self.z,
                direction=-self.direction,
                width=self.width,
                name="%s_sbend1" % self.name,
            )
            # sbend1.set_grid()
            sbend2 = sbend.Sbend(
                xlength=self.xlength_sbend,
                ylength=self.ylength_sbend,
                zlength=self.zlength,
                x=self.x + self.xlength_rectangle + self.xlength_trapezoid,
                y=self.y + self.ylength - self.ylength_sbend,
                z=self.z,
                direction=self.direction,
                width=self.width,
                name="%s_sbend2" % self.name,
            )
            # sbend2.set_grid()
        else:
            waveguide = Waveguide(
                xlength=self.xlength_rectangle,
                ylength=self.width,
                zlength=self.zlength,
                x=self.x + self.xlength_sbend + self.xlength_trapezoid,
                y=self.y + int(self.ylength / 2 - self.width / 2 + 0.5),
                z=self.z,
                width=self.width,
                name="%s_waveguide" % self.name,
            )
            # waveguide.set_grid()
            trapezoid = Trapezoid(
                xlength=self.xlength_trapezoid,
                ylength=self.ylength_trapezoid,
                zlength=self.zlength,
                x=self.x + self.xlength_sbend,
                y=self.y + int(self.ylength / 2 - self.ylength_trapezoid / 2 + 0.5),
                z=self.z,
                direction=self.direction,
                width=self.width,
                name="%s_trapezoid" % self.name,
            )
            # trapezoid.set_grid()
            sbend1 = sbend.Sbend(
                xlength=self.xlength_sbend,
                ylength=self.ylength_sbend,
                zlength=self.zlength,
                x=self.x,
                y=self.y,
                z=self.z,
                direction=-self.direction,
                width=self.width,
                name="%s_sbend1" % self.name,
            )
            # sbend1.set_grid()
            sbend2 = sbend.Sbend(
                xlength=self.xlength_sbend,
                ylength=self.ylength_sbend,
                zlength=self.zlength,
                x=self.x,
                y=self.y + self.ylength - self.ylength_sbend,
                z=self.z,
                direction=self.direction,
                width=self.width,
                name="%s_sbend2" % self.name,
            )
            # sbend2.set_grid()

        grid = fdtd.Grid(shape=(grid_xlength, grid_ylength, grid_zlength), grid_spacing=grid_spacing)

        grid[
            waveguide.x : waveguide.x + waveguide.xlength, waveguide.y : waveguide.y + waveguide.ylength
        ] = fdtd.Object(permittivity=waveguide.permittivity, name=waveguide.name)

        grid[
            trapezoid.x : trapezoid.x + trapezoid.xlength, trapezoid.y : trapezoid.y + trapezoid.ylength
        ] = fdtd.Object(permittivity=trapezoid.permittivity, name=trapezoid.name)

        grid[sbend1.x : sbend1.x + sbend1.xlength, sbend1.y : sbend1.y + sbend1.ylength] = fdtd.Object(
            permittivity=sbend1.permittivity, name=sbend1.name
        )

        grid[sbend2.x : sbend2.x + sbend2.xlength, sbend2.y : sbend2.y + sbend2.ylength] = fdtd.Object(
            permittivity=sbend2.permittivity, name=sbend2.name
        )

        grid[0:pml_width, :, :] = fdtd.PML(name="pml_xlow")
        grid[-pml_width:, :, :] = fdtd.PML(name="pml_xhigh")
        grid[:, 0:pml_width, :] = fdtd.PML(name="pml_ylow")
        grid[:, -pml_width:, :] = fdtd.PML(name="pml_yhigh")

        self._total_time = total_time
        self._grid = grid

    def set_source(self):
        self._grid[60:60, 35:45] = fdtd.LineSource(period=1550e-9 / 299792458, name="source")



from .waveguide import Waveguide
import numpy as np
import fdtd


class Sbend(Waveguide):

    """s波导代码，继承自waveguide
    xlength: 波导区域x方向宽度
    ylength: 波导区域y方向宽度
    zlength: 波导区域z方向宽度，通常也是波导高度
    x,y,z: 波导位置坐标（通常是矩形区域最靠近原点的点）
    direction: =1表示形状左上至右下，=-1表示形状从左下到右上
    width：波导宽度
    refractive_index:折射率"""

    def __init__(
        self,
        xlength=60,
        ylength=10,
        zlength=10,
        x=50,
        y=50,
        z=50,
        width=10,
        name="waveguide",
        refractive_index=1.7,
        direction=-1,
    ):
        self.direction = direction
        super().__init__(xlength, ylength, zlength, x, y, z, width, name, refractive_index)

    def _compute_permittivity(self):
        """
        输入波导规格，返回字典，包含名字、介电常数矩阵（规格为[ylength,xlength,zlength]）、区域规格、位置坐标、direction(=1表示形状左上至右下，=-1表示形状从左下到右上)
        """
        x = np.linspace(0, self.xlength, self.xlength)
        y = np.linspace(0, self.ylength, self.ylength)
        X, Y = np.meshgrid(x, y, indexing="ij")  # indexing = 'ij'很重要
        m = np.zeros((self.xlength, self.ylength, self.zlength))

        if self.direction == 1:
            # direction=1, 波导方向从左上到右下

            m1 = (
                Y
                <= 0.5 * (self.ylength - self.width) * np.sin((X / self.xlength - 0.5) * np.pi)
                + self.width / 2
                + self.ylength / 2
            )

            m2 = (
                Y
                >= 0.5 * (self.ylength - self.width) * np.sin((X / self.xlength - 0.5) * np.pi)
                - self.width / 2
                + self.ylength / 2
            )

        if self.direction == -1:
            # direction=-1, 波导方向从左下到右上
            m1 = (
                Y
                <= -0.5 * (self.ylength - self.width) * np.sin((X / self.xlength - 0.5) * np.pi)
                + self.width / 2
                + self.ylength / 2
            )
            m2 = (
                Y
                >= -0.5 * (self.ylength - self.width) * np.sin((X / self.xlength - 0.5) * np.pi)
                - self.width / 2
                + self.ylength / 2
            )

        for i in range(self.xlength):
            for j in range(self.ylength):
                if m1[i, j] == m2[i, j]:
                    m[i, j, :] = True

        permittivity = np.ones((self.xlength, self.ylength, self.zlength))
        permittivity += m[:, :] * (self.refractive_index**2 - 1)

        self.permittivity = permittivity

    def set_source(self):
        if self.direction == 1:
            self._grid[11:11, self.y : self.y + 10] = fdtd.LineSource(period=1550e-9 / 299792458, name="source")
        else:
            self._grid[11 + self.xlength : 11 + self.xlength, self.y : self.y + 10] = fdtd.LineSource(
                period=1550e-9 / 299792458, name="source"
            )



from waveguide import Waveguide
import numpy as np
import fdtd

# import os


class Ring(Waveguide):
    # TODO: 由于在设置的波导中，非波导部分折射率都为1，因此目前设置空间折射率来改变包层折射率并无意义
    """环形谐振腔，继承自Waveguide
    outer_radius: 外环半径
    zlength: 波导厚度
    x,y,z: 位置坐标（通常是矩形区域最靠近原点的点）
    width: 波导宽度
    l: 耦合长度
    gap: 环与直波导间距
    refractive_index:折射率
    name: 名称
    flag: 无意义
    """

    def __init__(
        self,
        outer_radius=60,
        zlength=10,
        x=50,
        y=50,
        z=50,
        width=5,
        l=0,
        gap=2,
        name="ring",
        refractive_index=3.47,
        flag=1,
    ):
        self.outer_r = outer_radius
        self.zlength = zlength
        self.x = x
        self.y = y
        self.z = z
        self.width = width
        self.name = name
        self.refractive_index = refractive_index
        self.l = l
        self.gap = 0
        self.flag = flag

    def set_ring(self):

        y = np.linspace(1, 2 * self.outer_r, 2 * self.outer_r)
        x = np.linspace(1, 2 * self.outer_r + self.l, 2 * self.outer_r + self.l)
        # TODO：把这个语句改成从1开始？
        X, Y = np.meshgrid(x, y, indexing="ij")  # indexing = 'ij'很重要

        if self.l == 0:

            m1 = (self.outer_r - self.width) ** 2 <= (X - self.outer_r) ** 2 + (Y - self.outer_r) ** 2
            m = (X - self.outer_r) ** 2 + (Y - self.outer_r) ** 2 <= self.outer_r**2

        for i in range(2 * self.outer_r + self.l):
            for j in range(2 * self.outer_r):
                if m[i, j] != m1[i, j]:
                    m[i, j] = 0

        else:

            m = np.zeros((self.outer_r * 2 + self.l, self.outer_r * 2, self.zlength))

            for j in range(2 * self.outer_r):
                for i in range(self.outer_r):

                    # 左半圆弧
                    if (self.outer_r - self.width) ** 2 <= (X[i, j] - self.outer_r) ** 2 + (
                        Y[i, j] - self.outer_r
                    ) ** 2 and (X[i, j] - self.outer_r) ** 2 + (Y[i, j] - self.outer_r) ** 2 <= self.outer_r**2:
                        m[i, j] = 1

                    if (self.outer_r - self.width) ** 2 <= (
                        X[self.outer_r + self.l + i, j] - self.outer_r - self.l
                    ) ** 2 + (Y[self.outer_r + self.l + i, j] - self.outer_r) ** 2 and (
                        X[self.outer_r + self.l + i, j] - self.outer_r - self.l
                    ) ** 2 + (
                        Y[self.outer_r + self.l + i, j] - self.outer_r
                    ) ** 2 <= self.outer_r**2:
                        m[self.outer_r + self.l + i, j] = 1

            for i in range(self.l):
                for j in range(self.width):
                    # 直波导
                    m[i + self.outer_r, j] = m1[i + self.outer_r, j] = 1
                    m[i + self.outer_r, 2 * self.outer_r - j - 1] = m1[i + self.outer_r, 2 * self.outer_r - j - 1] = 1

        permittivity = np.ones((self.outer_r * 2 + self.l, self.outer_r * 2, self.zlength))
        permittivity += m[:, :] * (self.refractive_index**2 - 1)

        result = {
            "name": self.name,
            "permittivity": permittivity,
            "size": (self.outer_r * 2 + self.l, self.outer_r * 2, self.zlength),
            "position": (self.x, self.y + self.width + self.gap, self.z),
            "flag": self.flag,
        }

        return result

    def set_wg(self):

        wg_bottom = Waveguide(
            xlength=self.outer_r * 2 + self.l,
            ylength=self.width,
            zlength=self.zlength,
            x=self.x,
            y=self.y,
            z=self.z,
            flag=self.flag,
            width=self.width,
            name="waveguide_bottom_%s" % self.name,
            refractive_index=self.refractive_index,
        )

        wg_top = Waveguide(
            xlength=self.outer_r * 2 + self.l,
            ylength=self.width,
            zlength=self.zlength,
            x=self.x,
            y=self.y + self.width + self.gap * 2 + self.outer_r * 2,
            z=self.z,
            flag=self.flag,
            width=self.width,
            name="waveguide_top_%s" % self.name,
            refractive_index=self.refractive_index,
        )

        result_top = wg_top.set_grid()
        result_bottom = wg_bottom.set_grid()

        return result_top, result_bottom


if __name__ == "__main__":

    ring = Ring(
        outer_radius=50,
        zlength=1,
        x=10,
        y=10,
        z=1,
        width=4,
        l=0,
        gap=1,
        name="ring",
        refractive_index=1.7,
        flag=1,
    )
    result_ring = ring.set_ring()
    result_top, result_bottom = ring.set_wg()
    # print(result_ring['name'])

    grid = fdtd.Grid(shape=(120, 130, 1), grid_spacing=155e-9)
    # permittivity=1.44 ** 2)

    grid[
        result_ring["position"][0] : result_ring["position"][0] + result_ring["size"][0],
        result_ring["position"][1] : result_ring["position"][1] + result_ring["size"][1],
    ] = fdtd.Object(permittivity=result_ring["permittivity"], name=result_ring["name"])

    grid[
        result_top["position"][0] : result_top["position"][0] + result_top["size"][0],
        result_top["position"][1] : result_top["position"][1] + result_top["size"][1],
    ] = fdtd.Object(permittivity=result_top["permittivity"], name=result_top["name"])

    grid[
        result_bottom["position"][0] : result_bottom["position"][0] + result_bottom["size"][0],
        result_bottom["position"][1] : result_bottom["position"][1] + result_bottom["size"][1],
    ] = fdtd.Object(permittivity=result_bottom["permittivity"], name=result_bottom["name"])

    PML_width = 5

    grid[0:PML_width, :, :] = fdtd.PML(name="pml_xlow")
    grid[-PML_width:, :, :] = fdtd.PML(name="pml_xhigh")
    grid[:, 0:PML_width, :] = fdtd.PML(name="pml_ylow")
    grid[:, -PML_width:, :] = fdtd.PML(name="pml_yhigh")

    grid[8:8, result_bottom["position"][1] - 1 : result_bottom["position"][1] + 6] = fdtd.LineSource(
        period=1550e-9 / 299792458, name="source"
    )
    # pulse = False,cycle=3, hanning_dt=4e-15)

    # grid[10:110, 10:120, 0] = fdtd.BlockDetector(name="detector")
    #
    # simfolder = grid.save_simulation("ring")  # initializing environment to save simulation data
    #
    # with open(os.path.join(simfolder, "grid.txt"), "w") as f:
    #     f.write(str(grid))
    #     wavelength = 1550e-9
    #     wavelengthUnits = wavelength / grid.grid_spacing
    #     GD = np.array([grid.x, grid.y, grid.z])
    #     gridRange = [np.arange(x / grid.grid_spacing) for x in GD]
    #     objectRange = np.array([[gridRange[0][x.x], gridRange[1][x.y], gridRange[2][x.z]] for x in grid.objects],
    #                            dtype=object).T
    #     f.write("\n\nGrid details (in wavelength scale):")
    #     f.write("\n\tGrid dimensions: ")
    #     f.write(str(GD / wavelength))
    #     # f.write("\n\tSource dimensions: ")
    #     # f.write(str(np.array([grid.source.x[-1] - grid.source.x[0] + 1, grid.source.y[-1] - grid.source.y[0] + 1, grid.source.z[-1] - grid.source.z[0] + 1])/wavelengthUnits))
    #     f.write("\n\tObject dimensions: ")
    #     f.write(str([(max(map(max, x)) - min(map(min, x)) + 1) / wavelengthUnits for x in objectRange]))

    grid.run(total_time=1000)
    # grid.save_data()

    # df = np.load(os.path.join(simfolder, "detector_readings.npz"))
    # fdtd.dB_map_2D(df["detector (E)"])
    grid.visualize(z=0, show=True)



import numpy as np
import fdtd
import matplotlib.pyplot as plt


class Waveguide:
    """
    xlength: 波导区域x方向宽度
    ylength: 波导区域y方向宽度
    zlength: 波导区域z方向宽度，通常也是波导高度
    x,y,z: 波导位置坐标（通常是矩形区域最靠近原点的点）
    width：波导宽度(在矩形波导中，波导宽度没有意义)
    refractive_index:折射率
    name:名称
    ！！！x，y仍然对应FDTD包中的x，y轴！！！
    """

    def __init__(
        self,
        xlength=60,
        ylength=10,
        zlength=10,
        x=50,
        y=50,
        z=50,
        width=10,
        name="waveguide",
        refractive_index=1.7,
    ):

        self.xlength = xlength
        self.ylength = ylength
        self.zlength = zlength
        self.x = x
        self.y = y
        self.z = z
        self.width = width
        self.name = name
        self.refractive_index = refractive_index

        self._grid = None  # 用来存储 fdtd.Grid 对象

        self._compute_permittivity()

    def _compute_permittivity(self):
        """矩形波导"""
        permittivity = np.zeros((self.xlength, self.ylength, self.zlength))
        permittivity += self.refractive_index**2

        self.permittivity = permittivity

    def set_grid(
        self, grid_xlength=100, grid_ylength=200, grid_zlength=50, grid_spacing=0.01, total_time=1, pml_width=10
    ):
        """
        Args:
            grid_xlength (int, optional): _description_. Defaults to 100.
            grid_ylength (int, optional): _description_. Defaults to 200.
            grid_zlength (int, optional): _description_. Defaults to 50.
            grid_spacing (float, optional): fdtd算法的空间步长（yee元胞的网格宽度）. Defaults to 0.01.
            total_time (int, optional): 计算时间. Defaults to 1.
            pml_width (int, optional): PML宽度. Defaults to 10.
        """

        grid = fdtd.Grid(shape=(grid_xlength, grid_ylength, grid_zlength), grid_spacing=grid_spacing)
        if grid_zlength != 1:
            grid[
                self.x : self.x + self.xlength,
                self.y : self.y + self.ylength,
                self.z : self.z + self.zlength,
            ] = fdtd.Object(permittivity=self.permittivity, name=self.name)
        else:
            grid[
                self.x : self.x + self.xlength,
                self.y : self.y + self.ylength,
            ] = fdtd.Object(permittivity=self.permittivity, name=self.name)

        grid[0:pml_width, :, :] = fdtd.PML(name="pml_xlow")
        grid[-pml_width:, :, :] = fdtd.PML(name="pml_xhigh")
        grid[:, 0:pml_width, :] = fdtd.PML(name="pml_ylow")
        grid[:, -pml_width:, :] = fdtd.PML(name="pml_yhigh")

        if grid_zlength != 1:
            grid[:, :, 0:pml_width] = fdtd.PML(name="pml_zlow")
            grid[:, :, -pml_width:] = fdtd.PML(name="pml_zhigh")

        self._total_time = total_time
        self._grid = grid

    def savefig(self, filepath, axis="x"):
        self._grid.run(total_time=self._total_time)

        if self._grid is None:
            raise RuntimeError("The grid should be set before saving figure.")

        axis = axis.lower()  # 识别大写的 "X"

        if axis == "x":  # 判断从哪一个轴俯视画图
            self._grid.visualize(x=0)
        elif axis == "y":
            self._grid.visualize(y=0)
        elif axis == "z":
            self._grid.visualize(z=0)
        else:
            raise RuntimeError("Unknown axis parameter.")

        plt.savefig(filepath)  # 保存图片
        plt.close()  # 清除画布



from waveguide import Waveguide
from ysplitter import Trapezoid


# import numpy as np
# import fdtd


class Mmi(Waveguide):
    """MMI，继承自Waveguide
    xlength: 多模波导区域x方向宽度
    ylength: 多模波导区域y方向宽度
    zlength: 多模波导区域z方向宽度，通常也是波导高度
    We: 多模波导有效宽度
    x,y,z: 多模波导位置坐标（通常是矩形区域最靠近原点的点）
    n: 输入波导数
    m：输出波导数
    width_port：端口宽度(端口是一个梯形，是一个Ysplitter.trapezoid类对象)
    width_wg:单模波导宽度
    ln：输入波导长度（x方向宽度）
    lm：输入波导长度（x方向宽度）
    dn: n*n MMI中，一组（一对）波导间的间距
    l_port: 端口长度
    refractive_index:折射率
    """

    def __init__(
        self,
        xlength=60,
        ylength=10,
        zlength=10,
        x=50,
        y=50,
        z=50,
        flag=1,
        width_wg=10,
        width_port=5,
        n=5,
        m=5,
        We=5,
        ln=10,
        lm=10,
        l_port=10,
        name="mmi",
        refractive_index=3.47,
        dn=1,
    ):
        Waveguide.__init__(self, xlength, ylength, zlength, x, y, z, flag, ylength, name, refractive_index)
        self.n = n
        self.m = m
        self.width_port = width_port
        self.width_wg = width_wg
        self.ln = ln
        self.lm = lm
        self.l_port = l_port
        self.We = We
        self.dn = dn

    def set_box(self):

        """设置多模波导"""

        waveguide0 = Waveguide(
            xlength=self.xlength,
            ylength=self.ylength,
            zlength=self.zlength,
            x=self.x,
            y=self.y,
            z=self.z,
            flag=self.flag,
            width=self.width,
            refractive_index=self.refractive_index,
            name="%s_waveguide0" % self.name,
        )
        result_waveguide0 = waveguide0.set_grid()
        return result_waveguide0

    def set_ports(self):

        """输入及输出端口"""

        x_port_in = [self.x - self.l_port] * self.n
        x_port_out = [self.x + self.xlength] * self.m
        y_port_out = [0] * self.m
        y_port_in = [0] * self.n

        if self.n == 1:

            y_port_in[0] = self.y + int(self.ylength / 2 - self.width_port / 2 + 0.5)

            for i in range(self.m):
                i += 1
                y_port_out[i - 1] = (
                    self.y
                    + int(self.ylength / 2 - self.width_port / 2 + 0.5)
                    + int(((2 * i - (self.m + 1)) / (2 * self.m)) * self.We + 0.5)
                )

        if self.n == 2:
            """2*m"""
            y_port_in[0] = self.y + int(self.ylength / 2 - self.We / 6 - self.width_port / 2 + 0.5)
            y_port_in[1] = self.y + int(self.ylength / 2 + self.We / 6 - self.width_port / 2 + 0.5)

            for i in range(self.m):
                i += 1
                y_port_out[i - 1] = (
                    self.y
                    + int(self.ylength / 2 - self.width_port / 2 + 0.5)
                    + int(((2 * i - (self.m + 1)) / (2 * self.m)) * self.We + 0.5)
                )

        if self.n > 2:
            """n*n"""
            if self.n % 2 == 0:  # n为偶 参考论文：bachmann 1994
                for i in range(int(self.n / 2)):
                    y_port_in[i * 2] = int(
                        self.y
                        + self.ylength
                        + self.dn / 2
                        - self.ylength / self.n
                        - i * 2 * self.ylength / self.n
                        + 0.5
                    )
                    y_port_in[i * 2 + 1] = y_port_in[i * 2] - self.dn - self.width_port
                    y_port_out[i * 2] = int(
                        self.y + self.ylength / self.n - self.dn / 2 - self.width_port + i * 2 * self.ylength / self.n
                    )
                    y_port_out[i * 2 + 1] = y_port_out[i * 2] + self.dn + self.width_port
            else:  # n为奇

                y_port_in = [0] * self.n

                for i in range(int(self.n / 2 - 0.5)):
                    y_port_in[i * 2] = int(
                        self.x + self.ylength + self.dn / 2 - self.ylength / self.n - i * 2 * self.ylength / self.n
                    )
                    y_port_in[i * 2 + 1] = y_port_in[i * 2] - self.dn - self.width_port
                    y_port_out[i * 2] = int(
                        self.x + self.ylength / self.n - self.dn / 2 - self.width_port + i * 2 * self.ylength / self.n
                    )
                    y_port_out[i * 2 + 1] = y_port_out[i * 2] + self.dn + self.width_port
                y_port_in[self.n - 1] = (
                    y_port_in[self.n - 2] - int(self.ylength * 2 / self.n) + self.dn + self.width_port
                )
                y_port_out[self.n - 1] = (
                    y_port_in[self.n - 2] + int(self.ylength * 2 / self.n) - self.dn - self.width_port
                )

        result_in_port = [0] * self.n
        result_out_port = [0] * self.m
        result_in_wg = [0] * self.n
        result_out_wg = [0] * self.m

        for i in range(self.n):
            port = Trapezoid(
                xlength=self.l_port,
                ylength=self.width_port,
                zlength=self.zlength,
                x=x_port_in[i],
                y=y_port_in[i],
                z=self.z,
                flag=1,
                width=self.width_wg,
                name="port_input%d" % i,
                refractive_index=self.refractive_index,
            )
            wg = Waveguide(
                xlength=self.ln,
                ylength=self.width_wg,
                zlength=self.zlength,
                x=x_port_in[i] - self.ln,
                y=y_port_in[i] + int(self.width_port / 2 - self.width_wg / 2 + 0.5),
                z=self.z,
                flag=1,
                width=self.width_wg,
                name="waveguide_input%d" % i,
                refractive_index=self.refractive_index,
            )
            result_in_port[i] = port.set_grid()
            result_in_wg[i] = wg.set_grid()

        for i in range(self.m):
            port = Trapezoid(
                xlength=self.l_port,
                ylength=self.width_port,
                zlength=self.zlength,
                x=x_port_out[i],
                y=y_port_out[i],
                z=self.z,
                flag=-1,
                width=self.width_port,
                name="port_output%d" % i,
                refractive_index=self.refractive_index,
            )
            wg = Waveguide(
                xlength=self.lm,
                ylength=self.width_wg,
                zlength=self.zlength,
                x=x_port_out[i] + self.l_port,
                y=y_port_out[i] + int(self.width_port / 2 - self.width_wg / 2 + 0.5),
                z=self.z,
                flag=1,
                width=self.width_wg,
                name="waveguide_output%d" % i,
                refractive_index=self.refractive_index,
            )
            result_out_port[i] = port.set_grid()
            result_out_wg[i] = wg.set_grid()

        return result_in_port, result_out_port, result_in_wg, result_out_wg


if __name__ == "__main__":
    mmi = Mmi(
        xlength=175,
        ylength=36,
        zlength=1,
        We=37,
        x=50,
        y=10,
        z=1,
        flag=1,
        name="mmi",
        refractive_index=3.47,
        n=1,
        m=2,
        width_port=8,
        width_wg=2,
        l_port=10,
        ln=40,
        lm=5,
    )

    result0 = mmi.set_box()
    result_in, result_out, result_in_wg, result_out_wg = mmi.set_ports()

    print(result_in[0]["permittivity"][:, :, 0])
    # print(result0['permittivity'][:,:,0])



# -*- coding: utf-8 -*-
from setuptools import setup

packages = \
['photfdtd']

package_data = \
{'': ['*']}

install_requires = \
['fdtd', 'matplotlib', 'numpy']

setup_kwargs = {
    'name': 'photfdtd',
    'version': '0.0.4',
    'description': 'PyPhot FDTD package',
    'long_description': '# PhotFDTD\n\nPhot FDTD 目前包含了基础器件和光器件的实现和测试用例，使用了fdtd库已经实现了的 Finite-difference time-domain method （时域有限差分）算法。\n\n## 基础器件\n\n1. waveguide - 直波导\n2. arc - 圆弧\n3. sbend - s波导\n4. ysplitter - y分支\n\n## 光器件\n\n光器件由基础器件组成，光器件与光器件之间也可以连接。\n\n1. directional coupler - 方向耦合器\n2. mmi - 多模耦合干涉仪\n3. ring - 微环\n\n## 安装\n\n在命令行界面输入以下命令即可\n\n```shell\npip install photfdtd\n```\n\n## Demo\n\n```python\nfrom photfdtd import Sbend\n\n\nif __name__ == "__main__":\n\n    # 设置器件参数\n    sbend = Sbend(\n        xlength=40, ylength=60, zlength=1, x=10, y=10, z=1, direction=-1, width=10, refractive_index=1.7, name="sbend"\n    )\n\n    # 设置 grid 参数\n    sbend.set_grid(grid_xlength=80, grid_ylength=80, grid_zlength=1, grid_spacing=155e-9, total_time=200, pml_width=10)\n\n    # 设置光源\n    sbend.set_source()\n\n    # 保存画好的图，设置保存位置，以及从哪一个轴俯视画图\n    sbend.savefig(filepath="SbendZ.png", axis="z")\n```\n\n### 运行结果\n\n![SbendZ](./docs/figures/SbendZ.png)\n\n## 各种光器件使用示例\n\n在 `tests` 目录下，可以看到各种光器件的使用示例，例如 `waveguide_test.py` 代表着波导的使用示例。\n\n## 开发者文档\n\n[开发者文档](docs/developer-guide.md) 提供了对于开发者的指导。',
    'author': 'Chunyu Li',
    'author_email': 'cyli0212@gmail.com',
    'maintainer': 'None',
    'maintainer_email': 'None',
    'url': 'https://pypi.org/project/photfdtd/',
    'packages': packages,
    'package_data': package_data,
    'install_requires': install_requires,
    'python_requires': '>=3.7,<4.0',
}


setup(**setup_kwargs)



import numpy as np
from .waveguide import Waveguide


class Arc(Waveguide):

    # TODO: 由于在设置的波导中，非波导部分折射率都为1，因此目前设置空间折射率来改变包层折射率并无意义
    """四分之一圆环
    outer_radius: 外环半径
    zlength: 波导厚度
    x,y,z: 位置坐标（通常是矩形区域最靠近原点的点）
    width: 波导宽度
    refractive_index:折射率
    name: 名称
    direction: 等于1，2，3，4，分别表征四个方向：左下，左上，右下，右上
    """

    def __init__(
        self,
        outer_radius: int,
        zlength: int,
        x: int,
        y: int,
        z: int,
        width: int,
        refractive_index: float,
        name: str,
        direction: int,
    ) -> None:

        self.direction = direction
        self.outer_radius = outer_radius
        super().__init__(outer_radius, outer_radius, zlength, x, y, z, width, name, refractive_index)

    def _compute_permittivity(self):
        x = y = np.linspace(1, self.outer_radius, self.outer_radius)
        X, Y = np.meshgrid(x, y, indexing="ij")  # indexing = 'ij'很重要

        if self.direction == 1:
            # direction=1, 圆心在左下
            m = (X - self.outer_radius) ** 2 + Y**2 >= (self.outer_radius - self.width) ** 2
            m1 = (X - self.outer_radius) ** 2 + Y**2 <= self.outer_radius**2

        elif self.direction == 2:
            # direction=2, 圆心在左上
            m = X**2 + Y**2 >= (self.outer_radius - self.width) ** 2
            m1 = X**2 + Y**2 <= self.outer_radius**2

        elif self.direction == 3:
            # direction=3, 圆心在右上
            m = X**2 + (Y - self.outer_radius) ** 2 >= (self.outer_radius - self.width) ** 2
            m1 = X**2 + (Y - self.outer_radius) ** 2 <= self.outer_radius**2

        elif self.direction == 4:
            # direction=4, 圆心在右下
            m = (X - self.outer_radius) ** 2 + (Y - self.outer_radius) ** 2 >= (self.outer_radius - self.width) ** 2
            m1 = (X - self.outer_radius) ** 2 + (Y - self.outer_radius) ** 2 <= self.outer_radius**2

        for i in range(self.outer_radius):
            for j in range(self.outer_radius):
                if m1[i, j] != m[i, j]:
                    m[i, j] = 0

        self.permittivity = np.ones((self.outer_radius, self.outer_radius, self.zlength))
        self.permittivity += m[:, :, None] * (self.refractive_index**2 - 1)
